/* ==============================================================================================================================================
 * LIBRARY:         Gestione VEML6075
 * FILE:            VEML6075.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         14/10/2016
 * UPDATED:         20/12/2016
 *
 * DESCRIP:         parametri e prototipi per libreria VEML6075.c
 *
 * ============================================================================================================================================== */

/* CONTROLLI DI COMPILAZIONE ==================================================================================================================== */

#ifndef I2C
    #error  Includere I2C_init.c
#endif

#ifdef VEML6075_I2C_MODULE_1                                                             // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
    #ifndef I2C_MODULE_1
        #error I2C_MODULE_1 non abilitato in I2C_init_settings.h
    #endif
    #define _VEML6075_I2C_1ByteAddr_2ByteData_write     I2C_1ByteAddr_2ByteData_write
    #define _VEML6075_I2C_1ByteAddr_2ByteData_read      I2C_1ByteAddr_2ByteData_read

#elif defined VEML6075_I2C_MODULE_2
    #ifndef I2C_MODULE_2
        #error I2C_MODULE_2 non abilitato in I2C_init_settings.h
    #endif
    #define _VEML6075_I2C_1ByteAddr_2ByteData_write     I2C2_1ByteAddr_2ByteData_write
    #define _VEML6075_I2C_1ByteAddr_2ByteData_read      I2C2_1ByteAddr_2ByteData_read

#else
    #error Definire il modulo I2C da utilizzare per VEML6075
#endif

/* INDIRIZZI REGISTRI =========================================================================================================================== */
#define VEML6075_ADDR                   0x20                                            // indirizzo VEML6075

#define VEML6075_CONFIG_REG             0x00
#define VEML6075_UVA_REG                0x07
#define VEML6075_UVB_REG                0x09
#define VEML6075_UVCOMP_1_REG           0x0A
#define VEML6075_UVCOMP_2_REG           0x0B
#define VEML6075_DEVICEID_REG           0x0C                

/* COSTANTI ===================================================================================================================================== */

#define VEML6075_RESET                  0x01                                            // reset to UV_IT: 50ms, HD: normal, UV_TRIG: 0, UV_AF: 0, SD: 1

#define VEML6075_SEQUENTIAL_AQ          0x00
#define VEML6075_ONE_AQ                 0x02

#define VEML6075_NORMAL_DR              0x00
#define VEML6075_HIGH_DR                0x08

#define VEML6075_IT_50MS                0x00
#define VEML6075_IT_100MS               0x10
#define VEML6075_IT_200MS               0x20
#define VEML6075_IT_400MS               0x30
#define VEML6075_IT_800MS               0x40

#define VEML6075_UV_TRIG                0x04

#define VEML6075_ACTIVE                 0x00
#define VEML6075_POWER_DOWN             0x01

/* PROTOTIPI E DOCUMENTAZIONE =================================================================================================================== */

/** @file VEML6075.c
 * Libreria per la gestione del sensore di raggi UVA e UVB Vishay VEML6075 via I2C
 * - Per un corretto funzionamento:
 *  + Includere la libreria I2C_init.c prima delle libreria VEML6075.c
 *  + Impostare il modulo I2C da utilizzare per il sensore VEML6075 in VEML6075_settings.h
 *  + Abilitare il modulo I2C utilizzato in I2C_init_settings.h
 *  + Avviare il modulo I2C nel programma principale richiamando la funzione I2C_init() o I2C2_init()
 * - Utilizzo:
 *  + Modalit continua:
 *    - Impostare la modalit di utilizzo tramite la funzione VEML6075_set(VEML6075_SEQUENTIAL_AQ,VEML6075_NORMAL_DR,VEML6075_IT_50MS);
 *    - Attivare il sensore tramite la funzione VEML6075_power_mode(VEML6075_ACTIVE);
 *    - Leggere i valori tramite le apposite funzioni 
 *  + Modalit One Shot:
 *    - Impostare la modalit di utilizzo tramite la funzione VEML6075_set(VEML6075_ONE_AQ,VEML6075_NORMAL_DR,VEML6075_IT_50MS);  
 *    - Attivare il sensore tramite la funzione VEML6075_power_mode(VEML6075_ACTIVE);
 *    - Avviare l'acquisizione tramite la funzione VEML6075_shot();
 *    - Attendere il tempo d'acquisizione (>150ms)
 *    - Leggere i valori tramite le apposite funzioni 
 *    - Per risparmiare corrente disattivare il sensore tramite la funzione VEML6075_power_mode(VEML6075_POWER_DOWN);
 * - Files:
 *  + VEML6075.c: Codice funzioni 
 *  + VEML6075.h: Parametri e prototipi
 *  + VEML6075_settings.h: Impostazioni specifiche per progetto
 *  - Note:
 *  I coefficienti e i calcoli sono presi dall'application note del 25/11/2016. Dopo varie prove risultano corretti i coefficienti per IT = 50ms. Teoricamente i coefficienti dovrebbero 
 *  dimezzarsi al raddoppiare dell'IT ma i dati ottenuti non risultano corretti. Con IT=50ms si possono rilevare valori massimi pi alti ma in complesso meno precisi. IT=100 o IT=200 darebbero 
 *  risultati migliori.
 */


/**
 * Resetta il sensore ai valori di default e lo imposta in power down mode
 * @param   void
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char VEML6075_reset (void);

/**
 * Imposta la modalit di acquisizione: continua o one acquisition, il range dinamico: normale o alto e il tempo di integrazione. Imposta il sensore in power down
 * @param   _mode: settaggio per la modalit di acquisizione (unsigned char):
 *              - VEML6075_SEQUENTIAL_AQ: acquisizione continua
 *              - VEML6075_ONE_AQ: acquisizione singola
 * @param   _dr: settaggio per il range dinamico (unsigned char):
 *              - VEML6075_NORMAL_DR
 *              - VEML6075_HIGH_DR
 * @param   _uv_it: setaggio per il tempo di integrazione (unsigned char):
 *              - VEML6075_IT_50MS
 *              - VEML6075_IT_100MS
 *              - VEML6075_IT_200MS
 *              - VEML6075_IT_400MS
 *              - VEML6075_IT_800MS
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision              
 */
signed char VEML6075_set(unsigned char _mode, unsigned char _dr, unsigned char _uv_it);

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_cal_responsivity
 * Input:       _uv_it: setaggio per il tempo di integrazione(VEML6075_IT_50MS, VEML6075_IT_100MS, VEML6075_IT_200MS, VEML6075_IT_400MS, VEML6075_IT_800MS)
 *
 * Overview:    
 * Nota:        Questa funzione va usata solo se prima non  stata usata la funzione VEML6075_set (il sensore  su un altro circuito)     
 * ============================================================================================================================================ */
/**
 * Calcola i coefficienti da utilizzare per i calcoli di UVA e UVB a seconda dell' IT selezionato
 * @param   _uv_it: setaggio per il tempo di integrazione (unsigned char):
 *              - VEML6075_IT_50MS
 *              - VEML6075_IT_100MS
 *              - VEML6075_IT_200MS
 *              - VEML6075_IT_400MS
 *              - VEML6075_IT_800MS
 * @note    Questa funzione va usata solo se prima non  stata usata la funzione VEML6075_set (il sensore  su un altro circuito)               
 */
void VEML6075_cal_responsivity (unsigned char _uv_it);

/**
 * Abilita o disabilita il sensore
 * @param    _mode: seleziona lo stato (unsigned char):
 *              - VEML6075_POWER_DOWN
 *              - VEML6075_ACTIVE
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char VEML6075_power_mode(unsigned char _mode);

/**
 * Imposta il trigger per l'acquisizione del dato ed attende che il dato sia stato acquisito
 * @param    void
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Il sensore deve essere in modalit VEML6075_ONE_AQ   \n
 *          Bloccante se il sensore non funziona correttamente
 */
signed char VEML6075_shot (void);

/**
 * Legge i valori acquisiti dal sensore
 * @param    void
 * @return  struct contenente i valori letti dal sensore (uva, uvb, uv_comp_1, uv_comp_2)
 * @note    In caso di errori di comunicazione potrebbero essere salvati i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
veml6075_uv_raw VEML6075_read_uv_raw (void);

/**
 * Converte i dati raw in ingresso in valori UVA UVB e UVI 
 * @param   struct contenente i valori letti dal sensore (uva, uvb, uv_comp_1, uv_comp_2)
 * @return  struct contenente i valori convertiti UVA in uW/cm^2, UVB in uW/cm^2, UVI (float)
 * @note    Se prima di usare questa funzione non  stata usata la funzione VEML6075_set impostare prima i coefficienti di calcolo tramite
 *          funzione VEML6075_cal_responsivity (caso in cui il sensore  su un altro circuito)
 */ 
veml6075_uv VEML6075_convert_uv_raw (veml6075_uv_raw _raw_input);

 /**
 * Legge i dati dal sensore e li converte in valori UVA UVB e UVI 
 * @param   void
 * @return  struct contenente i valori convertiti UVA in uW/cm^2, UVB in uW/cm^2, UVI (float)
 */ 
veml6075_uv VEML6075_read_uv (void);

/**
 * Legge l'id del dispositivo (0x0026)
 * @param   void
 * @return  Device ID (unsigned char)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char  VEML6075_read_device_id (void);
